#include "StdAfx.h"
#include "sakuext2.h"
#include "DialogMain.h"

#include "StringBuffer.h"
#include "RegKey.h"
#include "PathUtility.h"

CDialogMain::CDialogMain(void)
{
	_hwnd = NULL;
	_resourceName = MAKEINTRESOURCE(IDD_DIALOG_MAIN);
	_unicodeMode = false;
}

CDialogMain::~CDialogMain(void)
{
	if(_hwnd != NULL)
	{
		DestroyWindow(_hwnd);
		_hwnd = NULL;
	}
}

void CDialogMain::SetUnicodeMode(bool bFlag)
{
	_unicodeMode = bFlag;
}

void CDialogMain::SetSettingPath(LPCTSTR path)
{
	_settingPath = path;
}

void CDialogMain::SetExePath(LPCTSTR path)
{
	_exePath = path;
}

int CDialogMain::DoModal()
{
	_hwnd = CreateDialogParam(
		(HINSTANCE)GetModuleHandle(NULL),
	    _resourceName,
	    NULL,
	    DlgProcCommon,
		(LPARAM)this);

	if(_hwnd == NULL)
	{
		return -1;
	}
	MSG msg;
	while (GetMessage(&msg, NULL, 0, 0))
	{
		if (IsDialogMessage(_hwnd, &msg))
		{
			continue;
		}
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return (int) msg.wParam;

}

INT_PTR CDialogMain::OnInitDialog()
{
	RECT rect = {0};
	GetWindowRect(_hwnd, &rect);
	int x = GetSystemMetrics(SM_CXSCREEN) / 2 - (rect.right - rect.left) / 2;
	int y = GetSystemMetrics(SM_CYSCREEN) / 2 - (rect.bottom - rect.top) / 2;
	SetWindowPos(_hwnd, NULL, x, y, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
	ShowWindow(_hwnd, SW_SHOW);

	LoadSetting();

	return (INT_PTR)TRUE;
}

DWORD CDialogMain::LoadSetting()
{
	HWND hCombo = GetDlgItem(_hwnd, IDC_TYPE_LIST);
	ComboBox_AddString(hCombo, _TEXT("  (IĂ)"));
	ComboBox_SetCurSel(hCombo, 0);

	TCHAR sTypeName[256] = {0};
	TCHAR sTypeExts[4096] = {0};
	TCHAR sTypeLabel[256] = {0};
	char sAsciiPath[MAX_PATH] = {0};
#ifdef _UNICODE
	WideCharToMultiByte(CP_ACP, 0, _settingPath, (int)_tcslen(_settingPath), sAsciiPath, sizeof(sAsciiPath) - 1, NULL, NULL);
#else
	_tcsncpy_s(sAsciiPath, sizeof(sAsciiPath), _settingPath, (int)_tcslen(_settingPath));
#endif

	for(int iTypeIndex = 0; iTypeIndex < 20; iTypeIndex++)
	{

//		CMemoryArray<BYTE> sTypeNameAscii;
		char sTypeNameAscii[1024] = {0};
		CStringBuffer sTypeNameUnicode;
//		sTypeNameAscii.SetSize(1024);
//		sTypeNameAscii.ZeroClear();
		sTypeNameUnicode.SetBufferLength(1024);
		sTypeNameUnicode.FillZero();
		if(_unicodeMode)
		{
			char sTypes[100] = {0};
			sprintf_s(sTypes, sizeof(sTypes) / sizeof(sTypes[0]), "Types(%d)", iTypeIndex);
			GetPrivateProfileStringA(sTypes, "szTypeName", NULL, sTypeNameAscii, sizeof(sTypeNameAscii), sAsciiPath);

#ifdef _UNICODE
			MultiByteToWideChar(CP_UTF8, 0, sTypeNameAscii, sizeof(sTypeNameAscii), sTypeNameUnicode.GetBuffer(), sTypeNameUnicode.GetBufferLength());
#else
			_tcsncpy_s(sTypeNameUnicode.GetBuffer(), sTypeNameUnicode.GetBufferLength(), sTypeNameAscii, sizeof(sTypeNameAscii));
#endif
		}
		else
		{
			TCHAR sTypesTchar[100] = {0};
			_stprintf_s(sTypesTchar, sizeof(sTypesTchar) / sizeof(sTypesTchar[0]), _TEXT("Types(%d)"), iTypeIndex);
			GetPrivateProfileString(sTypesTchar, _TEXT("szTypeName"), NULL, sTypeNameUnicode.GetBuffer(), sTypeNameUnicode.GetBufferLength(), _settingPath);
		}

		if(_tcslen(sTypeNameUnicode) == 0)
		{
			break;
		}
		_stprintf_s(sTypeLabel, sizeof(sTypeLabel) / sizeof(sTypeLabel[0]), _TEXT("%2d:%s"), iTypeIndex + 1, sTypeNameUnicode.GetConstBuffer());
		ComboBox_AddString(hCombo, sTypeLabel);
	}

	return ERROR_SUCCESS;
}

int CDialogMain::RegistExt(LPCTSTR sExt, bool bDefProg)
{
	int errorCode = ERROR_SUCCESS;

	TCHAR sLowerExt[MAX_PATH] = {0};
	_tcsncpy_s(sLowerExt, sizeof(sLowerExt) / sizeof(sLowerExt[0]), sExt, _tcslen(sExt));
	CharLower(sLowerExt);

	TCHAR sDotExt[1024] = {_TEXT(".")};
	_tcsncat_s(sDotExt, sizeof(sDotExt) / sizeof(sDotExt[0]), sLowerExt, _tcslen(sLowerExt));

	CStringBuffer sDefExtType = _TEXT("SakuraEditor_");
	sDefExtType += sLowerExt;
	CRegKey regExt;
	CStringBuffer sExtType;
	sExtType.SetBufferLength(1024);

//	TCHAR sExtType[1024] = {_TEXT("SakuraEditor_")};
//	_tcsncat_s(sExtType, sizeof(sExtType) / sizeof(sExtType[0]), sLowerExt, _tcslen(sLowerExt));

	if(errorCode = regExt.Open(HKEY_CLASSES_ROOT, sDotExt, KEY_READ | KEY_WRITE))
	{
		if(errorCode = regExt.Create(HKEY_CLASSES_ROOT, sDotExt))
		{
			return errorCode;
		}
	}
	else
	{
		regExt.GetValue(NULL, sExtType.GetBuffer(), sExtType.GetBufferLength());
	}

	if(sExtType.GetLength() == 0)
	{
		if(errorCode = regExt.SetValue(NULL, sDefExtType))
		{
			return errorCode;
		}
		sExtType = sDefExtType;
	}

	CStringBuffer sShellActionCommandPath;
	sShellActionCommandPath.Append(sExtType);
	sShellActionCommandPath.Append(_TEXT("\\"));
	sShellActionCommandPath.Append(_TEXT("shell"));
	CStringBuffer sShellPath = sShellActionCommandPath;
	sShellActionCommandPath.Append(_TEXT("\\"));
	sShellActionCommandPath.Append(_TEXT("SakuraEditor"));
	CStringBuffer sShellActionPath = sShellActionCommandPath;
	sShellActionCommandPath.Append(_TEXT("\\"));
	sShellActionCommandPath.Append(_TEXT("command"));

	CRegKey keyShellActionCommand;
	if(errorCode = keyShellActionCommand.Create(HKEY_CLASSES_ROOT, sShellActionCommandPath, KEY_READ | KEY_WRITE) != ERROR_SUCCESS)
	{
		return errorCode;
	}
	CStringBuffer sCommandPathArg;
	sCommandPathArg.Append(_TEXT("\""));
	sCommandPathArg.Append(_exePath);
	sCommandPathArg.Append(_TEXT("\" \"%1\""));

	if(errorCode = keyShellActionCommand.SetValue(NULL, sCommandPathArg))
	{
		return errorCode;
	}

	CRegKey keyShellAction;
	if(errorCode = keyShellAction.Create(HKEY_CLASSES_ROOT, sShellActionPath, KEY_READ | KEY_WRITE) != ERROR_SUCCESS)
	{
		return errorCode;
	}
	if(errorCode = keyShellAction.SetValue(NULL, _TEXT("Sakura &Editor")))
	{
		return errorCode;
	}

	if(bDefProg)
	{
		CRegKey keyShell;
		if(errorCode = keyShell.Create(HKEY_CLASSES_ROOT, sShellPath, KEY_READ | KEY_WRITE) != ERROR_SUCCESS)
		{
			return errorCode;
		}
		if(errorCode = keyShell.SetValue(NULL, _TEXT("SakuraEditor")))
		{
			return errorCode;
		}
	}

	return 0;
}

int CDialogMain::UnregistExt(LPCTSTR sExt)
{
	int errorCode = ERROR_SUCCESS;

	TCHAR sLowerExt[MAX_PATH] = {0};
	_tcsncpy_s(sLowerExt, sizeof(sLowerExt) / sizeof(sLowerExt[0]), sExt, _tcslen(sExt));
	CharLower(sLowerExt);

	TCHAR sDotExt[1024] = {_TEXT(".")};
	_tcsncat_s(sDotExt, sizeof(sDotExt) / sizeof(sDotExt[0]), sLowerExt, _tcslen(sLowerExt));

	CRegKey regExt;
	TCHAR sExtType[1024] = {0};

	if(regExt.Open(HKEY_CLASSES_ROOT, sDotExt) != ERROR_SUCCESS)
	{
		return ERROR_SUCCESS;
	}
	if(regExt.GetValue(NULL, sExtType, sizeof(sExtType) / sizeof(sExtType[0])) != ERROR_SUCCESS)
	{
		return ERROR_SUCCESS;
	}

	if(_tcslen(sExtType) == 0)
	{
		return ERROR_SUCCESS;
	}

	const int nBufferSize = 1024;
	CStringBuffer sShellActionCommandPath;
	sShellActionCommandPath.Append(sExtType);
	sShellActionCommandPath.Append(_TEXT("\\"));
	sShellActionCommandPath.Append(_TEXT("shell"));
	CStringBuffer sShellPath = sShellActionCommandPath;
	sShellActionCommandPath.Append(_TEXT("\\"));
	sShellActionCommandPath.Append(_TEXT("SakuraEditor"));
	CStringBuffer sShellActionPath = sShellActionCommandPath;
	sShellActionCommandPath.Append(_TEXT("\\"));
	sShellActionCommandPath.Append(_TEXT("command"));

	CRegKey keyShell;
	if(keyShell.Open(HKEY_CLASSES_ROOT, sShellPath, KEY_READ | KEY_WRITE) == ERROR_SUCCESS)
	{

		CStringBuffer defAction;
		defAction.SetBufferLength(nBufferSize + 1);
		if(keyShell.GetValue(NULL, defAction.GetBuffer(), nBufferSize) == ERROR_SUCCESS)
		{
			if(defAction.Equals(_TEXT("SakuraEditor")))
			{
				if(errorCode = keyShell.DeleteValue(NULL))
				{
					return errorCode;
				}
			}
		}

		if(CRegKey::ExistsKey(HKEY_CLASSES_ROOT, sShellActionCommandPath))
		{
			if(errorCode = CRegKey::DeleteKey(HKEY_CLASSES_ROOT, sShellActionCommandPath))
			{
				return errorCode;
			}
		}
		if(CRegKey::ExistsKey(HKEY_CLASSES_ROOT, sShellActionPath))
		{
			if(errorCode = CRegKey::DeleteKey(HKEY_CLASSES_ROOT, sShellActionPath))
			{
				return errorCode;
			}
		}
	}

	regExt.Close();

	CStringBuffer defaultExtType = _TEXT("SakuraEditor_");
	defaultExtType += sLowerExt;
	if(defaultExtType.Equals(sExtType))
	{
		int nActionIndex = 0;
		CStringBuffer sActionTemp;
		sActionTemp.SetBufferLength(nBufferSize + 1);
		if(keyShell.EnumKey(nActionIndex, sActionTemp.GetBuffer(), nBufferSize) == ERROR_NO_MORE_ITEMS)
		{
			keyShell.Close();
			if(errorCode = CRegKey::DeleteKey(HKEY_CLASSES_ROOT, sShellPath))
			{
				return errorCode;
			}
			if(errorCode = CRegKey::DeleteKey(HKEY_CLASSES_ROOT, sExtType))
			{
				return errorCode;
			}
		}
	}
	
	if(!CRegKey::ExistsKey(HKEY_CLASSES_ROOT, sExtType))
	{
		if(errorCode = regExt.Open(HKEY_CLASSES_ROOT, sDotExt, KEY_READ | KEY_WRITE))
		{
			return errorCode;
		}
		if(errorCode = regExt.DeleteValue(NULL))
		{
			return errorCode;
		}

		int nActionIndex = 0;
		CStringBuffer sActionTemp;
		sActionTemp.SetBufferLength(nBufferSize + 1);
		if(regExt.EnumKey(nActionIndex, sActionTemp.GetBuffer(), nBufferSize) == ERROR_NO_MORE_ITEMS)
		{
			regExt.Close();
			if(errorCode = CRegKey::DeleteKey(HKEY_CLASSES_ROOT, sDotExt))
			{
				return errorCode;
			}
		}
	}
	return ERROR_SUCCESS;
}


void CDialogMain::OnUnregistClick()
{
//	CMemoryArray<TCHAR> sTypeExts;
	CStringBuffer sTypeExts;
	int sTypeExtsLen = Edit_GetTextLength(GetDlgItem(_hwnd, IDC_EXTS));
	if(sTypeExtsLen == 0)
	{
		MessageBox(_hwnd, _TEXT("gq͂ĂB"), _TEXT("G["), MB_OK);
		return;
	}

	if(MessageBox(_hwnd, _TEXT("WXgo^܂H"), _TEXT("mF"), MB_OKCANCEL) != IDOK)
	{
		return;
	}

//	sTypeExts.SetSize(sTypeExtsLen + 1);
//	sTypeExts.ZeroClear();
	sTypeExts.SetBufferLength(sTypeExtsLen + 1);
	sTypeExts.FillZero();
	GetDlgItemText(_hwnd, IDC_EXTS, sTypeExts.GetBuffer(), sTypeExts.GetBufferLength());

	for(int nExt = 0; nExt < sTypeExts.GetLength() - 1; nExt++)
	{
		if(sTypeExts[nExt] == '.')
		{
			continue;
		}
		const TCHAR* pSep = _tcsstr(sTypeExts.GetBuffer(nExt), _TEXT(","));
		if(pSep == NULL)
		{
			pSep = sTypeExts.GetBuffer(sTypeExts.GetLength());
		}
		int nSepLen = (int)(pSep - sTypeExts.GetBuffer(nExt));
		CStringBuffer sExt;
		sExt.SetBufferLength(nSepLen + 1);
//		CMemoryArray<TCHAR> sExt;
//		sExt.SetSize(nSepLen + 1);
		_tcsncpy_s(sExt.GetBuffer(), sExt.GetBufferLength(), sTypeExts.GetConstBuffer(nExt), nSepLen);
//		CharLower(sExt.GetBuffer());
//		MessageBox(_hwnd, sExt.GetConstBuffer(), _TEXT("debug"), MB_OK);
		if(UnregistExt(sExt.GetConstBuffer()))
		{
			MessageBox(_hwnd, _TEXT("o^Ɏs܂"), _TEXT("G["), MB_OK);
			return;
		}
		nExt += nSepLen;
	}
	MessageBox(_hwnd, _TEXT("o^܂"), _TEXT(""), MB_OK);
}

void CDialogMain::OnRegistClick()
{
//	CMemoryArray<TCHAR> sTypeExts;
	int sTypeExtsLen = Edit_GetTextLength(GetDlgItem(_hwnd, IDC_EXTS));
	if(sTypeExtsLen == 0)
	{
		MessageBox(_hwnd, _TEXT("gq͂ĂB"), _TEXT("G["), MB_OK);
		return;
	}

	if(MessageBox(_hwnd, _TEXT("WXgɓo^܂H"), _TEXT("mF"), MB_OKCANCEL) != IDOK)
	{
		return;
	}

	CStringBuffer sTypeExts;
	sTypeExts.SetBufferLength(sTypeExtsLen + 1);
//	sTypeExts.SetSize(sTypeExtsLen + 1);
//	sTypeExts.ZeroClear();
	sTypeExts.FillZero();
	GetDlgItemText(_hwnd, IDC_EXTS, sTypeExts.GetBuffer(), sTypeExts.GetBufferLength());

	bool bDefProg =(IsDlgButtonChecked(_hwnd, IDC_CHECK_DEFAPP) == BST_CHECKED);
	for(int nExt = 0; nExt < sTypeExts.GetLength() - 1; nExt++)
	{
		if(sTypeExts[nExt] == '.')
		{
			continue;
		}
		const TCHAR* pSep = _tcsstr(sTypeExts.GetBuffer(nExt), _TEXT(","));
		if(pSep == NULL)
		{
			pSep = sTypeExts.GetBuffer(sTypeExts.GetLength());
		}
		int nSepLen = (int)(pSep - sTypeExts.GetBuffer(nExt));
		CStringBuffer sExt;
		sExt.SetBufferLength(nSepLen + 1);
		_tcsncpy_s(sExt.GetBuffer(), sExt.GetBufferLength(), sTypeExts.GetConstBuffer(nExt), nSepLen);
//		MessageBox(_hwnd, sExt.GetConstBuffer(), _TEXT("debug"), MB_OK);
		if(RegistExt(sExt.GetConstBuffer(), bDefProg))
		{
			MessageBox(_hwnd, _TEXT("o^Ɏs܂"), _TEXT("G["), MB_OK);
			return;
		}
		nExt += nSepLen;
	}
	MessageBox(_hwnd, _TEXT("o^܂"), _TEXT(""), MB_OK);
}

void CDialogMain::OnTypeListChanged()
{
	HWND hCombo = GetDlgItem(_hwnd, IDC_TYPE_LIST);
	TCHAR sTypeLabel[256] = {0};
	ComboBox_GetText(hCombo, sTypeLabel, sizeof(sTypeLabel) / sizeof(sTypeLabel[0]));
	int nIndex = -1;
	_stscanf_s(sTypeLabel, _TEXT("%2d:"), &nIndex);
	if(nIndex < 0)
	{
		return;
	}
	TCHAR sTypes[100] = {0};
	_stprintf_s(sTypes, sizeof(sTypes) / sizeof(sTypes[0]), _TEXT("Types(%d)"), nIndex - 1);
	TCHAR sTypeExts[4096] = {0};
	GetPrivateProfileString(sTypes, _TEXT("szTypeExts"), NULL, sTypeExts, sizeof(sTypeExts), _settingPath);
	SetDlgItemText(_hwnd, IDC_EXTS, sTypeExts);
}

void CDialogMain::OnVersionClick()
{
	CStringBuffer message;
	message += _TEXT("TNGfB^pgq֘Atc[\n");
	message += _TEXT("sakuext2 Ver 1.0\n\n");
	message += _TEXT("Copyright 2008 Beta.Ito\n");
	message += _TEXT("The zlib/libpng license\n\n");
	message += _TEXT("Build:");
	message += _TEXT(__DATE__);
	message += _TEXT("\n");
	if(_unicodeMode)
	{
		message += _TEXT("Mode:Unicode");
	}
	else
	{
		message += _TEXT("Mode:Ascii");
	}
	MessageBox(_hwnd, message, _TEXT("o[W"), MB_OK | MB_ICONINFORMATION);
}

INT_PTR CDialogMain::OnCommand(WPARAM wParam, LPARAM lParam)
{
	if (LOWORD(wParam) == IDC_BUTTON_REGIST)
	{
		OnRegistClick();
		return (INT_PTR)TRUE;
	}
	if (LOWORD(wParam) == IDC_BUTTON_UNREGIST)
	{
		OnUnregistClick();
		return (INT_PTR)TRUE;
	}
	if (LOWORD(wParam) == IDC_BUTTON_VERSION)
	{
		OnVersionClick();
		return (INT_PTR)TRUE;
	}
	if (LOWORD(wParam) == IDC_TYPE_LIST)
	{
		if(HIWORD(wParam) == CBN_SELCHANGE)
		{
			OnTypeListChanged();
		}
		return (INT_PTR)TRUE;
	}
	
	if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL)
	{
		EndDialog(_hwnd, LOWORD(wParam));
		return (INT_PTR)TRUE;
	}
	return (INT_PTR)FALSE;
}

INT_PTR CDialogMain::OnClose()
{
	PostQuitMessage(0);
	return (INT_PTR)FALSE;
}

INT_PTR CDialogMain::OnDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		return OnInitDialog();
	case WM_COMMAND:
		return OnCommand(wParam, lParam);
	case WM_CLOSE:
		return OnClose();
	}
	return (INT_PTR)FALSE;
}

INT_PTR CALLBACK CDialogMain::DlgProcCommon(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	CDialogMain* pDialog = (CDialogMain*)(LONG_PTR)GetWindowLongPtr(hwnd, DWLP_USER);
	if(pDialog == NULL)
	{
		if(message == WM_INITDIALOG)
		{
			pDialog = (CDialogMain*)lParam;
			pDialog->_hwnd = hwnd;
			SetWindowLongPtr(hwnd, DWLP_USER, (LONG)reinterpret_cast<LONG_PTR>(pDialog));
		}
		else if(message == WM_CLOSE)
		{
			PostQuitMessage(0);
		}
		else
		{
			return (INT_PTR)TRUE;
		}
	}
	return pDialog->OnDlgProc(hwnd, message, wParam, lParam);
}
