#pragma once

#include <tchar.h>

class CStringBuffer
{
protected:
	TCHAR* _buffer;
	int _size;
public:
	CStringBuffer(void)
	{
		_buffer = NULL;
		_size = 0;
	}
	
	CStringBuffer(const TCHAR* source)
	{
		_buffer = NULL;
		_size = 0;
		Append(source);
	}
	
	CStringBuffer(const CStringBuffer& source)
	{
		_buffer = NULL;
		_size = 0;
		Append(source);
	}
	
	virtual ~CStringBuffer(void)
	{
		delete[] _buffer;
		_buffer = NULL;
	}

	TCHAR* GetBuffer(int index = 0)
	{
		return _buffer + index;
	}

	const TCHAR* GetConstBuffer(int index = 0) const
	{
		return _buffer + index;
	}

	int GetLength() const
	{
		return (int)_tcslen(_buffer);
	}

	int GetBufferLength() const
	{
		return _size;
	}

	CStringBuffer& operator=(const TCHAR* source)
	{
		delete[] _buffer;
		_buffer = NULL;
		Append(source);
		return *this;
	}

	CStringBuffer& operator=(const CStringBuffer& source)
	{
		delete[] _buffer;
		_buffer = NULL;
		Append(source);
		return *this;
	}

	TCHAR& operator[](int index)
	{
		return _buffer[index];
	}
	operator const TCHAR*() const
	{
		return _buffer;
	}
	CStringBuffer& operator+=(const TCHAR* source)
	{
		Append(source);
		return *this;
	}
	CStringBuffer operator+(const TCHAR* source)
	{
		CStringBuffer buffer2;
		buffer2.Append(GetConstBuffer());
		buffer2.Append(source);
		return buffer2;
	}
/*
	void SetLength(int len)
	{
		TCHAR* pNewBuffer = new TCHAR[len + 1];
		if(_buffer)
		{
			_tcsncpy_s(pNewBuffer, len, _buffer, min(_size, len));
			delete[] _buffer;
		}
		else
		{
			pNewBuffer[0] = 0;
			pNewBuffer[len] = 0;
		}

		_buffer = pNewBuffer;
		_size = len + 1;
	}
*/
	void SetBufferLength(int len)
	{
		TCHAR* pNewBuffer = new TCHAR[len];
		if(_buffer)
		{
			_tcsncpy_s(pNewBuffer, len, _buffer, min(_size, len));
			delete[] _buffer;
		}
		else
		{
			pNewBuffer[0] = 0;
		}
		if(len > 0)
		{
			pNewBuffer[len - 1] = 0;
		}

		_buffer = pNewBuffer;
		_size = len;
	}

	void FillZero()
	{
		if(_buffer)
		{
			ZeroMemory(_buffer, sizeof(TCHAR) * _size);
		}
	}

	void Append(const TCHAR* source)
	{
		if(!source)
		{
			return;
		}
		size_t srclen = _tcslen(source);
		TCHAR* pNewBuffer = new TCHAR[_size + srclen + 1];
		if(_buffer)
		{
			_tcsncpy_s(pNewBuffer, _size + srclen + 1, _buffer, _size);
		}
		else
		{
			pNewBuffer[0] = 0;
		}
		_tcsncat_s(pNewBuffer, _size + srclen + 1, source, srclen);
		delete[] _buffer;
		_buffer = pNewBuffer;
		_size += (int)(srclen + 1);
	}

	bool Equals(const TCHAR* source)
	{
		if(!_buffer && (source == NULL || _tcslen(source) == 0))
		{
			return true;
		}
		return (StrCmp(_buffer, source) == 0);
	}

	void ToLower()
	{
		if(_buffer == NULL)
		{
			return;
		}
		CharLower(_buffer);
	}
};
